
`define	HSMB_LVDS_ss_clkin_p		hsmc_clk_in_p[2]  //hsmc_clk_in_p[2]
`define	HSMB_LVDS_ss_clkout_p		hsmc_clk_out_p[2] //hsmc_clk_out_p[2]

`timescale 1 ps / 1 ps
module  bts_hsmc_test (
		
        input         clk_50_max10, 
        input         clk_25_max10,
        input         clk_10_adc,
        input         clk_lvds_125_p,		
        input         clk_ddr3_100_p,
        input         fpga_resetn, 
        
        //LED PB DIPSW
        output [4:0]  user_led,
        input  [3:0]  user_pb,
        input  [4:0]  user_dipsw,
        
        //HSMC
        input  [2:1]  hsmc_clk_in_p,
        input         hsmc_clk_in0,
        output [2:1]  hsmc_clk_out_p,
        input  [16:0] hsmc_rx_d_p,
        output [16:0] hsmc_tx_d_p
        );

    parameter         CH_NUM = 17;

	//LVDS instance 
    wire [135:0]      tx_data;
    wire [135:0]      tx_data_temp;
    reg  [135:0]      tx_data_sync;
    wire [135:0]      rx_data;
    reg  [135:0]      rx_data_sync;
    wire              rx_outclock;
    wire              tx_coreclock;
    wire [CH_NUM-1:0] rx_channel_data_align/* synthesis keep */;
    wire [CH_NUM-1:0] rx_channel_data_align_qsys;
    wire [CH_NUM-1:0] rx_channel_data_align_auto;
    wire [CH_NUM-1:0] pll_areset/* synthesis keep */;
    wire              qsys_align;
    wire              tx_locked/* synthesis keep */;
    wire              rx_locked/* synthesis keep */;
    wire              rx_cda_reset;
    
    //Qsys test interface  
    wire [39:0]       tx_data_x40_g3/* synthesis keep */;
    wire [39:0]       rx_data_x40_g3/* synthesis keep */;
    wire              data_pattern_valid_x40_g3/* synthesis keep */;
    reg               data_pattern_valid_x40_g3_reg1/* synthesis keep */;
    reg               data_pattern_valid_x40_g3_reg2/* synthesis keep */;
    wire              data_pattern_ready_x40_g3/* synthesis keep */;
    reg               data_pattern_ready_x40_g3_reg1/* synthesis keep */;
    reg               data_pattern_ready_x40_g3_reg2/* synthesis keep */;
    
    wire [31:0]       tx_data_x32_g2/* synthesis keep */;
    wire [31:0]       rx_data_x32_g2/* synthesis keep */;
    wire              data_pattern_valid_x32_g2/* synthesis keep */;
    reg               data_pattern_valid_x32_g2_reg1/* synthesis keep */;
    reg               data_pattern_valid_x32_g2_reg2/* synthesis keep */;
    wire              data_pattern_ready_x32_g2/* synthesis keep */;
    reg               data_pattern_ready_x32_g2_reg1/* synthesis keep */;
    reg               data_pattern_ready_x32_g2_reg2/* synthesis keep */;
    
    wire [31:0]       tx_data_x32_g1/* synthesis keep */;
    wire [31:0]       rx_data_x32_g1/* synthesis keep */;
    wire              data_pattern_valid_x32_g1/* synthesis keep */;
    reg               data_pattern_valid_x32_g1_reg1/* synthesis keep */;
    reg               data_pattern_valid_x32_g1_reg2/* synthesis keep */;
    wire              data_pattern_ready_x32_g1/* synthesis keep */;
    reg               data_pattern_ready_x32_g1_reg1/* synthesis keep */;
    reg               data_pattern_ready_x32_g1_reg2/* synthesis keep */;
    
    wire [31:0]       tx_data_x32_g0/* synthesis keep */;
    wire [31:0]       rx_data_x32_g0/* synthesis keep */;
    wire              data_pattern_valid_x32_g0/* synthesis keep */;
    reg               data_pattern_valid_x32_g0_reg1/* synthesis keep */;
    reg               data_pattern_valid_x32_g0_reg2/* synthesis keep */;
    wire              data_pattern_ready_x32_g0/* synthesis keep */;
    reg               data_pattern_ready_x32_g0_reg1/* synthesis keep */;
    reg               data_pattern_ready_x32_g0_reg2/* synthesis keep */;

	wire [CH_NUM-1:0] sync_valid;		// Sync LVDS success
    reg               pb_align_1;
    reg               pb_align_2;
    reg               pb_align_3;
    wire              pb_align_p;
	 
    assign user_led[0]    = !tx_locked;
    assign user_led[1]    = !rx_locked;
    assign user_led[4:2]  = 3'b111;
        
    assign tx_data[135:0] = {tx_data_x32_g2, tx_data_x32_g1, tx_data_x32_g0, tx_data_x40_g3};
    assign rx_data_x32_g2 = rx_data[135:104];
    assign rx_data_x32_g1 = rx_data[103:72];
    assign rx_data_x32_g0 = rx_data[71:40];
    assign rx_data_x40_g3 = rx_data[39:0];
	
    assign rx_cda_reset   = !fpga_resetn;

    always @(posedge rx_outclock or negedge fpga_resetn)
    begin
      if(!fpga_resetn)
         begin
         data_pattern_valid_x32_g0_reg1 <= 1'b0;
         data_pattern_valid_x32_g0_reg2 <= 1'b0;
         data_pattern_valid_x32_g1_reg1 <= 1'b0;
         data_pattern_valid_x32_g1_reg2 <= 1'b0;
         data_pattern_valid_x32_g2_reg1 <= 1'b0;
         data_pattern_valid_x32_g2_reg2 <= 1'b0;
         data_pattern_valid_x40_g3_reg1 <= 1'b0;
         data_pattern_valid_x40_g3_reg2 <= 1'b0;
         end
      else
         begin
         data_pattern_valid_x32_g0_reg1 <= data_pattern_valid_x32_g0;
         data_pattern_valid_x32_g0_reg2 <= data_pattern_valid_x32_g0_reg1;
         data_pattern_valid_x32_g1_reg1 <= data_pattern_valid_x32_g1;
         data_pattern_valid_x32_g1_reg2 <= data_pattern_valid_x32_g1_reg1;
         data_pattern_valid_x32_g2_reg1 <= data_pattern_valid_x32_g2;
         data_pattern_valid_x32_g2_reg2 <= data_pattern_valid_x32_g2_reg1;
         data_pattern_valid_x40_g3_reg1 <= data_pattern_valid_x40_g3;
         data_pattern_valid_x40_g3_reg2 <= data_pattern_valid_x40_g3_reg1;
         end
    end
    
    always @(posedge tx_coreclock or negedge fpga_resetn)
    begin
      if(!fpga_resetn)
         begin
         data_pattern_ready_x32_g0_reg1 <= 1'b0;
         data_pattern_ready_x32_g0_reg2 <= 1'b0;
         data_pattern_ready_x32_g1_reg1 <= 1'b0;
         data_pattern_ready_x32_g1_reg2 <= 1'b0;
         data_pattern_ready_x32_g2_reg1 <= 1'b0;
         data_pattern_ready_x32_g2_reg2 <= 1'b0;
         data_pattern_ready_x40_g3_reg1 <= 1'b0;
         data_pattern_ready_x40_g3_reg2 <= 1'b0;
         end
      else
         begin
         data_pattern_ready_x32_g0_reg1 <= data_pattern_ready_x32_g0;
         data_pattern_ready_x32_g0_reg2 <= data_pattern_ready_x32_g0_reg1;
         data_pattern_ready_x32_g1_reg1 <= data_pattern_ready_x32_g1;
         data_pattern_ready_x32_g1_reg2 <= data_pattern_ready_x32_g1_reg1;
         data_pattern_ready_x32_g2_reg1 <= data_pattern_ready_x32_g2;
         data_pattern_ready_x32_g2_reg2 <= data_pattern_ready_x32_g2_reg1;
         data_pattern_ready_x40_g3_reg1 <= data_pattern_ready_x40_g3;
         data_pattern_ready_x40_g3_reg2 <= data_pattern_ready_x40_g3_reg1;
         end
    end
    
    serdes_tx_x17  	serdes_tx_x17_inst	(
                                        .pll_areset             (pll_areset             ),
                                        .tx_inclock             (clk_lvds_125_p         ),
                                        .tx_in                  (tx_data_sync           ),
                                        .tx_out                 (hsmc_tx_d_p[16:0]      ),
                                        .tx_outclock            (`HSMB_LVDS_ss_clkout_p ),
                                        .tx_locked              (tx_locked              ),
                                        .tx_coreclock           (tx_coreclock           )
                                        );

	serdes_rx_x17	serdes_rx_x17_inst (
                                        .pll_areset             (pll_areset             ),
                                        .rx_inclock             (clk_lvds_125_p         ), //`HSMB_LVDS_ss_clkin_p  ),
                                        .rx_in                  (hsmc_rx_d_p[16:0]      ),
                                        .rx_out                 (rx_data                ),
                                        .rx_outclock            (rx_outclock            ),
                                        .rx_locked              (rx_locked              ),
                                        .rx_channel_data_align  (rx_channel_data_align  ),
                                        .rx_cda_reset           ({17{rx_cda_reset}}     )
                                        );

    xcvr_test_system    xcvr_test_system (
                                        .clk_50_clk                                        (clk_50_max10                  ), //                                        clk_50.clk
                                        .reset_50_reset_n                                  (fpga_resetn                   ), //                                      reset_50.reset_n
                                        .pio_0_external_connection_export                  (rx_channel_data_align_qsys    ), //                     pio_0_external_connection.export
                                        .pio_1_external_connection_export                  (pll_areset                    ), //                     pio_1_external_connection.export
                                        .pio_2_external_connection_export                  (qsys_align                    ), //                     pio_2_external_connection.export
                                        .data_pattern_checker_x32_g0_pattern_in_valid      (data_pattern_valid_x32_g0_reg2), //        data_pattern_checker_x32_g0_pattern_in.valid
                                        .data_pattern_checker_x32_g0_pattern_in_ready      (data_pattern_ready_x32_g0     ), //                                              .ready
                                        .data_pattern_checker_x32_g0_pattern_in_data       (rx_data_x32_g0                ), //                                              .data
                                        .data_pattern_checker_x32_g0_pattern_in_clk_clk    (rx_outclock                   ), //    data_pattern_checker_x32_g0_pattern_in_clk.clk
                                        .data_pattern_generator_x32_g0_pattern_out_valid   (data_pattern_valid_x32_g0     ), //     data_pattern_generator_x32_g0_pattern_out.valid
                                        .data_pattern_generator_x32_g0_pattern_out_ready   (data_pattern_ready_x32_g0_reg2), //                                              .ready
                                        .data_pattern_generator_x32_g0_pattern_out_data    (tx_data_x32_g0                ), //                                              .data
                                        .data_pattern_generator_x32_g0_pattern_out_clk_clk (tx_coreclock                  ), // data_pattern_generator_x32_g0_pattern_out_clk.clk
                                        .data_pattern_checker_x32_g1_pattern_in_valid      (data_pattern_valid_x32_g1_reg2), //        data_pattern_checker_x32_g1_pattern_in.valid
                                        .data_pattern_checker_x32_g1_pattern_in_ready      (data_pattern_ready_x32_g1     ), //                                              .ready
                                        .data_pattern_checker_x32_g1_pattern_in_data       (rx_data_x32_g1                ), //                                              .data
                                        .data_pattern_checker_x32_g1_pattern_in_clk_clk    (rx_outclock                   ), //    data_pattern_checker_x32_g1_pattern_in_clk.clk
                                        .data_pattern_generator_x32_g1_pattern_out_valid   (data_pattern_valid_x32_g1     ), //     data_pattern_generator_x32_g1_pattern_out.valid
                                        .data_pattern_generator_x32_g1_pattern_out_ready   (data_pattern_ready_x32_g1_reg2), //                                              .ready
                                        .data_pattern_generator_x32_g1_pattern_out_data    (tx_data_x32_g1                ), //                                              .data
                                        .data_pattern_generator_x32_g1_pattern_out_clk_clk (tx_coreclock                  ), // data_pattern_generator_x32_g1_pattern_out_clk.clk
                                        .data_pattern_checker_x32_g2_pattern_in_valid      (data_pattern_valid_x32_g2_reg2), //        data_pattern_checker_x32_g2_pattern_in.valid
                                        .data_pattern_checker_x32_g2_pattern_in_ready      (data_pattern_ready_x32_g2     ), //                                              .ready
                                        .data_pattern_checker_x32_g2_pattern_in_data       (rx_data_x32_g2                ), //                                              .data
                                        .data_pattern_checker_x32_g2_pattern_in_clk_clk    (rx_outclock                   ), //    data_pattern_checker_x32_g2_pattern_in_clk.clk
                                        .data_pattern_generator_x32_g2_pattern_out_valid   (data_pattern_valid_x32_g2     ), //     data_pattern_generator_x32_g2_pattern_out.valid
                                        .data_pattern_generator_x32_g2_pattern_out_ready   (data_pattern_ready_x32_g2_reg2), //                                              .ready
                                        .data_pattern_generator_x32_g2_pattern_out_data    (tx_data_x32_g2                ), //                                              .data
                                        .data_pattern_generator_x32_g2_pattern_out_clk_clk (tx_coreclock                  ), // data_pattern_generator_x32_g2_pattern_out_clk.clk
                                        .data_pattern_checker_x40_g3_pattern_in_valid      (data_pattern_valid_x40_g3_reg2), //        data_pattern_checker_x40_g3_pattern_in.valid
                                        .data_pattern_checker_x40_g3_pattern_in_ready      (data_pattern_ready_x40_g3     ), //                                              .ready
                                        .data_pattern_checker_x40_g3_pattern_in_data       (rx_data_x40_g3                ), //                                              .data
                                        .data_pattern_checker_x40_g3_pattern_in_clk_clk    (rx_outclock                   ), //    data_pattern_checker_x40_g3_pattern_in_clk.clk
                                        .data_pattern_generator_x40_g3_pattern_out_valid   (data_pattern_valid_x40_g3     ), //     data_pattern_generator_x40_g3_pattern_out.valid
                                        .data_pattern_generator_x40_g3_pattern_out_ready   (data_pattern_ready_x40_g3_reg2), //                                              .ready
                                        .data_pattern_generator_x40_g3_pattern_out_data    (tx_data_x40_g3                ), //                                              .data
                                        .data_pattern_generator_x40_g3_pattern_out_clk_clk (tx_coreclock                  )  // data_pattern_generator_x40_g3_pattern_out_clk.clk
                                        );

//------------------------------------------------------
//LVDS Channel Alignment Management
//------------------------------------------------------
//Filter push button jitter
always @ ( posedge tx_coreclock or negedge fpga_resetn )
begin
	if ( !fpga_resetn )
	begin
    pb_align_1 <= #2 1'b0;
    pb_align_2 <= #2 1'b0;
    pb_align_3 <= #2 1'b0;			
	end
	else
	begin
    pb_align_1 <= #2 !user_pb[0] | qsys_align; // posedge is valid
    pb_align_2 <= #2 pb_align_1;
    pb_align_3 <= #2 pb_align_2;	
	end
end	

//Generate the pulse from cnt
rise_edge   sync_rise(
            .clk     (rx_outclock ),
            .rst_n   (fpga_resetn ),
            .din     (pb_align_3  ),
            .dout    (pb_align_p  )
            );

always @(posedge rx_outclock or negedge fpga_resetn)
begin
    if(!fpga_resetn)
       begin
       rx_data_sync <= {136{1'b0}};
       end
    else
       begin 
       rx_data_sync <= rx_data;
       end
end

//RX data align
genvar i;
generate for ( i=0; i<CH_NUM; i=i+1 )
begin: inst_align
align_top   u_align(
            .clk            (rx_outclock                  ),
            .rst_n          (fpga_resetn                  ),
            .tx_locked      (tx_locked                    ),
            .rx_locked      (rx_locked                    ),
            .rx_data        (rx_data_sync[8*i+7:8*i]      ),
            .pb_align_p     (pb_align_p                   ),
            //Output
            .align_bit      (rx_channel_data_align_auto[i]),
            .sync_valid     (sync_valid[i]                )
            );
end
endgenerate

assign rx_channel_data_align = rx_channel_data_align_auto | rx_channel_data_align_qsys;
	
genvar j;
generate for ( j=0; j<CH_NUM; j=j+1 )
begin: inst_tx	
assign tx_data_temp[8*j+7:8*j] = ( sync_valid[j] == 1'b0 ) ? {8'hf0} : tx_data[8*j+7:8*j];	
end
endgenerate

always @(posedge tx_coreclock or negedge fpga_resetn)
begin
    if(!fpga_resetn)
       begin
       tx_data_sync <= {136{1'b0}};
       end
    else
       begin 
       tx_data_sync <= tx_data_temp;
       end
end

endmodule
