/*****************************************************************************
 *  File: reconfig_utils.h
 *
 *  This file contains the constants, structure definitions, and funtion
 *  prototypes for the application selector.
 *
 ****************************************************************************/

#ifndef APP_SELECTOR_H_
#define APP_SELECTOR_H_

#include "sys/alt_flash_dev.h"

//#define DEBUG_MODE 1 // debugging help
//#define MAN_DEBUG 1 // debugging help
//#define WEB_PAGE_ONLY_MODE 1

#define NUM_VIDEO_FRAMES 2

#define SREC_BUFFER_SIZE             0x00020000

#define FLASH_SECTOR_SIZE            0x10000  // 64KB
#define FLASH_BOOT_BLOCK_SECTOR_SIZE 0x10000  // 64KB
#define BOARD_ID 					 0x2A

/******************************************************************
// Max10 DevKit
 Section                        Size(KB)    Address
 ------------------------------------------------------------------
 Board test system scratch      512         0x03F80000 - 0x03FFFFFF
 User software                  56640       0x00830000 - 0x03F7FFFF
 Factory software               4096        0x00430000 - 0x0082FFFF
 Zips(html, web content)        4096        0x00030000 - 0x0042FFFF
 ------------------------------------------------------------------
 Board information              64          0x00020000 - 0x0002FFFF
 Ethernet option bits           64          0x00010000 - 0x0001FFFF
 User design reset vector       64          0x00000000 - 0x0000FFFF
*******************************************************************/
#define FACTORY_SW_IMAGE_OFFSET      0x00430000
#define FACTORY_WP_IMAGE_OFFSET      0x00030000

// correct user #defines
#define USER_SW_IMAGE_OFFSET         0x00830000

#define RO_ZIPFS_OFFSET              0x00030000
#define BOARD_INFO_OFFSET            0x00020000
#define ETHERNET_OPTION_BITS         0x00010000

// These are the SIVGX SI 16bit masks for the MAXII reconfiguation 16bit registers:
#define MAXII_CLEAR_PSR_MASK         0xFFFE // 1111 1111 1111 1110 | 0
#define MAXII_CLEAR_PSO_MASK         0xFFFD // 1111 1111 1111 1101 | 1
#define MAXII_FORCE_RECONFIG_MASK    0x000B // 0000 0000 0000 1011 | B
#define MAXII_CLEAR_PSS_MASK         0xFFF7 // 1111 1111 1111 0111 | 3

//#define MAXII_RSU_REG				0x6 // was 0x8
#define MAXII_RSU_REG  0x8 			// MAXII_REG2 is 0x8

#define PAGE_USER				1	// NIOS_PIO_PAGE_SELECT_BASE
#define SRST					0	// NIOS_PIO_RECONFIGURE_N_BASE

#define USE_HAL_FLASH_DRIVER            0
//#define USE_INTEL_P30_FLASH_DRIVER      0
//#define USE_INTEL_G18_FLASH_DRIVER      1

// Error Codes
#define SUCCESS                      0
#define FAILURE                     -1
#define ERROR_FAT_FAILED_TO_MOUNT   -2
#define ERROR_FAT_COULD_NOT_FIND    -3
#define ERROR_FAT_READ_ZERO_BYTES   -4
#define ERROR_SREC_HAS_NO_TIMESTAMP -5
#define ERROR_COULD_NOT_ALLOCATE    -6

// Filetypes
#define FLASH                        1
#define RBF                          2

#define FILE_BUFFER_SIZE FLASH_SECTOR_SIZE

int ReconfigFPGA(unsigned int hw_flash_page);

#endif /*APP_SELECTOR_H_*/
